<?php
/**
 * Plugin Name: Способ оплаты ДелоБанк
 * Description: Плагин, позволяющий проводить платежи по картам Visa и MasterCard
 * Author: DeloBank
 * Version: 1.0.4
 */

include_once __DIR__ . '/paymentid.php';

class DeloBank
{
    private static $pluginName = 'delobank';
    private static $lang = []; // массив с переводом плагина
    private static $path = '';
    private static $options = '';

    public function __construct()
    {
        self::$path = PLUGIN_DIR . self::$pluginName;
    }

    /**
     * Метод выполняющийся при активации плагина
     */
    public static function activate()
    {
        $paymentId = self::getPayment();
        if (empty($paymentId)) {
            $paymentId = self::createPayment();
            $fileExist = self::addFile();
            if ($paymentId && $fileExist) {
                return true;
            }
        }
        return false;
    }

    /**
     * Проверяет существует ли способ оплаты ДелоБанк, тем самым проверяет установлен ли плагин
     * @return int|null
     */
    private static function getPayment()
    {
        $q = 'SELECT `id` FROM `' . PREFIX . 'payment` WHERE `id` = ' . DELOBANK_PAYMENT_ID;
        $dbRes = DB::query($q);
        if ($result = DB::fetchAssoc($dbRes)) {
            return (int)$result['id'];
        }
        return null;
    }

    /**
     * Создает в БД запись с нашим способом оплаты
     * @return int
     */
    private static function createPayment()
    {
        $q = 'INSERT INTO `' . PREFIX . 'payment` (
                `name`,
                `paramArray`,
                `sort`,
                `permission`
            ) VALUES (
                "Способ оплаты ДелоБанк",
                "",
                0,
                "all"
            )';
        if (DB::query($q)) {
            $paymentId = (int)DB::insertId();
            self::savePaymentIdInFile($paymentId);
            $settingFields = self::settingFields();
            $defaultSetting = self::defaultSetting();
            $param = [];
            foreach ($settingFields as $field) {
                if (isset($defaultSetting[$field])) {
                    $value = str_replace('#paymentId#', $paymentId, $defaultSetting[$field]);
                    $value = CRYPT::mgCrypt($value);
                } else {
                    $value = '';
                }
                $param[$field] = $value;
            }
            $paramJson = json_encode($param);
            $q = 'UPDATE `' . PREFIX . "payment` SET
                    `paramArray` = '" . $paramJson . "'
                WHERE `id` = " . $paymentId;
            DB::query($q);
            self::assocDeliveryPayment($paymentId);
            return $paymentId;
        }
        return 0;
    }

    /**
     * Добавляет информацию при каком способе доставке возможен наш способ оплаты
     * @param int $paymentId id способа оплаты
     * @return bool
     */
    private static function assocDeliveryPayment($paymentId)
    {
        $q = 'INSERT INTO `' . PREFIX . 'delivery_payment_compare` (
                `payment_id`,
                `delivery_id`,
                `compare`
            ) VALUES (' . $paymentId . ', 1, 1), (' . $paymentId . ', 2, 1), (' . $paymentId . ', 3, 1)';
        return DB::query($q);
    }

    /**
     * Сохраняет id способа оплаты в файл
     * @param int $paymentId id способа оплаты
     * @return bool|int
     */
    private static function savePaymentIdInFile($paymentId)
    {
        return file_put_contents(
            __DIR__ . '/paymentid.php',
            "<?php\ndefine('DELOBANK_PAYMENT_ID', " . $paymentId . ");\n",
            LOCK_EX
        );
    }

    /**
     * Все возможные поля настройки
     * @return array
     */
    private static function settingFields()
    {
        return [
            'merch_id',
            'purchase_account_amount_id',
            'back_url_s',
            'back_url_f',
            'url',
            'lang',
            'currency',
        ];
    }

    private static function defaultSetting()
    {
        return [
            'back_url_s'         => 'https://' . $_SERVER['HTTP_HOST'] . '/personal',
            'back_url_f'         => 'https://' . $_SERVER['HTTP_HOST'] . '/personal',
            'url'                => isset($_SERVER['DELOBANK_ENVIRONMENT']) ? 'https://testmpi.myskb.ru/payment/start.wsm' : 'https://mpi.skbbank.ru/payment/start.wsm',
            'lang'               => 'RU',
            'currency'           => 643,
        ];
    }

    /**
     * Метод выполняется при деактивации плагина
     */
    public static function deActivate()
    {
        $paymentId = self::getPayment();
        $deleteOk = self::deletePayment($paymentId);
        $fileOk = self::delFile();
        if ($deleteOk && $fileOk) {
            return true;
        }
        return false;
    }

    /**
     * Удаляет способ оплаты из БД
     * @param int $paymentId
     * @return bool
     */
    private static function deletePayment($paymentId)
    {
        $q = 'DELETE FROM `' . PREFIX . 'payment` WHERE `id` = ' . $paymentId;
        return DB::query($q);
    }

    /**
     * Копирует файл с подключением формы оплаты в нужную папку
     * @return bool
     */
    private static function addFile()
    {
        $file = __DIR__ . '/payment_';
        $fileDest = __DIR__ . '/../../mg-core/layout/payment_';
        return copy($file, $fileDest);
    }

    /**
     * Удаляем наш костыльный файл
     * @return bool
     */
    private static function delFile()
    {
        $file = __DIR__ . '/../../mg-core/layout/payment_';
        return unlink($file);
    }
}

mgActivateThisPlugin(__FILE__, ['DeloBank', 'activate']);
mgDeactivateThisPlugin(__FILE__, ['DeloBank', 'deActivate']);
